@@ -143,12 +143,15 @@ def settings(*args, **kwargs):
 
 def cd(path):
     """
-    Context manager that keeps directory state when calling operations.
+    Context manager that keeps directory state when calling remote operations.
 
     Any calls to `run`, `sudo`, `get`, or `put` within the wrapped block will
     implicitly have a string similar to ``"cd <path> && "`` prefixed in order
-    to give the sense that there is actually statefulness involved.  `cd` only
-    affects the remote paths for `get` and `put` -- local paths are untouched.
+    to give the sense that there is actually statefulness involved.
+
+    .. note::
+        `cd` only affects *remote* paths -- to modify *local* paths, use
+        `~fabric.context_managers.lcd`.
 
     Because use of `cd` affects all such invocations, any code making use of
     those operations, such as much of the ``contrib`` section, will also be
@@ -195,6 +198,8 @@ def cd(path):
     .. versionchanged:: 1.0
         Applies to `get` and `put` in addition to the command-running
         operations.
+
+    .. seealso:: `~fabric.context_managers.lcd`
     """
     return _change_cwd('cwd', path)
 
