@@ -1,8 +1,15 @@
-from nose.tools import eq_
+from nose.tools import eq_, ok_
 from fudge import Fake, with_fakes
+import random
 
-from fabric import decorators
+from fabric import decorators, tasks
+from fabric.state import env
 
+def test_task_returns_an_instance_of_wrappedfunctask_object():
+    def foo():
+        pass
+    task = decorators.task(foo)
+    ok_(isinstance(task, tasks.WrappedCallableTask))
 
 def fake_function(*args, **kwargs):
     """
@@ -38,3 +45,13 @@ def test_runs_once_returns_same_value_each_run():
     task = decorators.runs_once(fake_function().returns(return_value))
     for i in range(2):
         eq_(task(), return_value)
+
+def test_with_settings_passes_env_vars_into_decorated_function():
+    env.value = True
+    random_return = random.randint(1000, 2000)
+    def some_task():
+        return env.value
+    decorated_task = decorators.with_settings(value=random_return)(some_task)
+    ok_(some_task(), msg="sanity check")
+    eq_(random_return, decorated_task())
+
