@@ -3,20 +3,24 @@ from __future__ import with_statement
 import os
 import shutil
 import sys
-import tempfile
 import types
 from contextlib import nested
 from StringIO import StringIO
 
+import unittest
+import random
+import types
+
 from nose.tools import raises, eq_
 from fudge import with_patched_object
 
-from fabric.state import env
+from fabric.state import env, output
 from fabric.operations import require, prompt, _sudo_prefix, _shell_wrap, \
     _shell_escape
-from fabric.api import get, put, hide, show, cd, lcd, local
+from fabric.api import get, put, hide, show, cd, lcd, local, run, sudo
 from fabric.sftp import SFTP
 
+from fabric.decorators import with_settings
 from utils import *
 from server import (server, PORT, RESPONSES, FILES, PASSWORDS, CLIENT_PRIVKEY,
     USER, CLIENT_PRIVKEY_PASSPHRASE)
@@ -25,6 +29,7 @@ from server import (server, PORT, RESPONSES, FILES, PASSWORDS, CLIENT_PRIVKEY,
 # require()
 #
 
+
 def test_require_single_existing_key():
     """
     When given a single existing key, require() throws no exceptions
@@ -80,6 +85,30 @@ def test_require_mixed_state_keys_prints_missing_only():
         assert 'foo' in err
 
 
+@mock_streams('stderr')
+@raises(SystemExit)
+def test_require_iterable_provided_by_key():
+    """
+    When given a provided_by iterable value, require() raises SystemExit
+    """
+    # 'version' is one of the default values, so we know it'll be there
+    def fake_providing_function():
+        pass
+    require('foo', provided_by=[fake_providing_function])
+
+
+@mock_streams('stderr')
+@raises(SystemExit)
+def test_require_noniterable_provided_by_key():
+    """
+    When given a provided_by noniterable value, require() raises SystemExit
+    """
+    # 'version' is one of the default values, so we know it'll be there
+    def fake_providing_function():
+        pass
+    require('foo', provided_by=fake_providing_function)
+
+
 #
 # prompt()
 #
@@ -87,6 +116,7 @@ def test_require_mixed_state_keys_prints_missing_only():
 def p(x):
     print x,
 
+
 @mock_streams('stdout')
 @with_patched_object(sys.modules['__builtin__'], 'raw_input', p)
 def test_prompt_appends_space():
@@ -108,7 +138,7 @@ def test_prompt_with_default():
     d = "default!"
     prompt(s, default=d)
     eq_(sys.stdout.getvalue(), "%s [%s] " % (s, d))
-    
+
 
 #
 # run()/sudo()
@@ -131,6 +161,7 @@ def test_sudo_prefix_without_user():
     eq_(_sudo_prefix(user=None), env.sudo_prefix % env.sudo_prompt)
 
 
+@with_settings(use_shell=True)
 def test_shell_wrap():
     prefix = "prefix"
     command = "command"
@@ -149,6 +180,7 @@ def test_shell_wrap():
         del eq_.description
 
 
+@with_settings(use_shell=True)
 def test_shell_wrap_escapes_command_if_shell_is_true():
     """
     _shell_wrap() escapes given command if shell=True
@@ -192,33 +224,60 @@ def test_shell_escape_escapes_backticks():
     eq_(_shell_escape(cmd), "touch test.pid && kill \`cat test.pid\`")
 
 
+class TestCombineStderr(FabricTest):
+    @server()
+    def test_local_none_global_true(self):
+        """
+        combine_stderr: no kwarg => uses global value (True)
+        """
+        output.everything = False
+        r = run("both_streams")
+        # Note: the exact way the streams are jumbled here is an implementation
+        # detail of our fake SSH server and may change in the future.
+        eq_("ssttddoeurtr", r.stdout)
+        eq_(r.stderr, "")
+
+    @server()
+    def test_local_none_global_false(self):
+        """
+        combine_stderr: no kwarg => uses global value (False)
+        """
+        output.everything = False
+        env.combine_stderr = False
+        r = run("both_streams")
+        eq_("stdout", r.stdout)
+        eq_("stderr", r.stderr)
+
+    @server()
+    def test_local_true_global_false(self):
+        """
+        combine_stderr: True kwarg => overrides global False value
+        """
+        output.everything = False
+        env.combine_stderr = False
+        r = run("both_streams", combine_stderr=True)
+        eq_("ssttddoeurtr", r.stdout)
+        eq_(r.stderr, "")
+
+    @server()
+    def test_local_false_global_true(self):
+        """
+        combine_stderr: False kwarg => overrides global True value
+        """
+        output.everything = False
+        env.combine_stderr = True
+        r = run("both_streams", combine_stderr=False)
+        eq_("stdout", r.stdout)
+        eq_("stderr", r.stderr)
+
 #
 # get() and put()
 #
 
 class TestFileTransfers(FabricTest):
-    def setup(self):
-        super(TestFileTransfers, self).setup()
-        self.tmpdir = tempfile.mkdtemp()
-
-    def teardown(self):
-        super(TestFileTransfers, self).teardown()
-        shutil.rmtree(self.tmpdir)
-
-    def path(self, *path_parts):
-        return os.path.join(self.tmpdir, *path_parts)
-
-    def exists_remotely(self, path):
-        return SFTP(env.host_string).exists(path)
-
-    def exists_locally(self, path):
-        return os.path.exists(path)
-
-
     #
     # get()
     #
-
     @server(files={'/home/user/.bashrc': 'bash!'}, home='/home/user')
     def test_get_relative_remote_dir_uses_home(self):
         """
@@ -228,8 +287,6 @@ class TestFileTransfers(FabricTest):
             # Another if-it-doesn't-error-out-it-passed test; meh.
             eq_(get('.bashrc', self.path()), [self.path('.bashrc')])
 
-
-
     @server()
     def test_get_single_file(self):
         """
@@ -241,7 +298,6 @@ class TestFileTransfers(FabricTest):
             get(remote, local)
         eq_contents(local, FILES[remote])
 
-
     @server()
     def test_get_sibling_globs(self):
         """
@@ -253,7 +309,6 @@ class TestFileTransfers(FabricTest):
         for remote in remotes:
             eq_contents(self.path(remote), FILES[remote])
 
-
     @server()
     def test_get_single_file_in_folder(self):
         """
@@ -264,7 +319,6 @@ class TestFileTransfers(FabricTest):
             get('folder', self.tmpdir)
         eq_contents(self.path(remote), FILES[remote])
 
-
     @server()
     def test_get_tree(self):
         """
@@ -276,7 +330,6 @@ class TestFileTransfers(FabricTest):
         for path, contents in leaves:
             eq_contents(self.path(path[1:]), contents)
 
-
     @server()
     def test_get_tree_with_implicit_local_path(self):
         """
@@ -296,7 +349,6 @@ class TestFileTransfers(FabricTest):
             if os.path.exists(dirname):
                 shutil.rmtree(dirname)
 
-
     @server()
     def test_get_absolute_path_should_save_relative(self):
         """
@@ -309,7 +361,6 @@ class TestFileTransfers(FabricTest):
         assert self.exists_locally(os.path.join(lpath, 'subfolder'))
         assert not self.exists_locally(os.path.join(lpath, 'tree/subfolder'))
 
-
     @server()
     def test_path_formatstr_nonrecursively_is_just_filename(self):
         """
@@ -321,7 +372,6 @@ class TestFileTransfers(FabricTest):
             get('/tree/subfolder/file3.txt', ltarget)
         assert self.exists_locally(os.path.join(lpath, 'file3.txt'))
 
-
     @server()
     @mock_streams('stderr')
     def _invalid_file_obj_situations(self, remote_path):
@@ -341,7 +391,6 @@ class TestFileTransfers(FabricTest):
         """
         self._invalid_file_obj_situations('/tree')
 
-
     @server()
     def test_get_single_file_absolutely(self):
         """
@@ -352,7 +401,6 @@ class TestFileTransfers(FabricTest):
             get(target, self.tmpdir)
         eq_contents(self.path(os.path.basename(target)), FILES[target])
 
-
     @server()
     def test_get_file_with_nonexistent_target(self):
         """
@@ -364,7 +412,6 @@ class TestFileTransfers(FabricTest):
             get(target, local)
         eq_contents(local, FILES[target])
 
-
     @server()
     @mock_streams('stderr')
     def test_get_file_with_existing_file_target(self):
@@ -380,7 +427,6 @@ class TestFileTransfers(FabricTest):
         assert "%s already exists" % local in sys.stderr.getvalue()
         eq_contents(local, FILES[target])
 
-
     @server()
     def test_get_file_to_directory(self):
         """
@@ -394,7 +440,6 @@ class TestFileTransfers(FabricTest):
             get(target, self.tmpdir)
         eq_contents(self.path(target), FILES[target])
 
-
     @server(port=2200)
     @server(port=2201)
     def test_get_from_multiple_servers(self):
@@ -419,7 +464,6 @@ class TestFileTransfers(FabricTest):
                         tmp, "127.0.0.1-%s" % port, 'file3.txt'
                     ))
 
-
     @server()
     def test_get_from_empty_directory_uses_cwd(self):
         """
@@ -432,7 +476,6 @@ class TestFileTransfers(FabricTest):
         for x in "file.txt file2.txt tree/file1.txt".split():
             assert os.path.exists(os.path.join(self.tmpdir, x))
 
-
     @server()
     def _get_to_cwd(self, arg):
         path = 'file.txt'
@@ -461,7 +504,6 @@ class TestFileTransfers(FabricTest):
         """
         self._get_to_cwd(None)
 
-
     @server()
     def test_get_should_accept_file_like_objects(self):
         """
@@ -473,7 +515,6 @@ class TestFileTransfers(FabricTest):
             get(target, fake_file)
         eq_(fake_file.getvalue(), FILES[target])
 
-
     @server()
     def test_get_interpolation_without_host(self):
         """
@@ -490,7 +531,6 @@ class TestFileTransfers(FabricTest):
             get('/folder/file3.txt', local_path)
             assert self.exists_locally(tmp + "bar/file3.txt")
 
-
     @server()
     def test_get_returns_list_of_local_paths(self):
         """
@@ -502,7 +542,6 @@ class TestFileTransfers(FabricTest):
         files = ['file1.txt', 'file2.txt', 'subfolder/file3.txt']
         eq_(map(lambda x: os.path.join(d, 'tree', x), files), retval)
 
-
     @server()
     def test_get_returns_none_for_stringio(self):
         """
@@ -511,18 +550,17 @@ class TestFileTransfers(FabricTest):
         with hide('everything'):
             eq_([], get('/file.txt', StringIO()))
 
-
     @server()
     def test_get_return_value_failed_attribute(self):
         """
-        get()'s return value should indicate any paths which failed to download.
+        get()'s return value should indicate any paths which failed to
+        download.
         """
         with settings(hide('everything'), warn_only=True):
             retval = get('/doesnt/exist', self.path())
         eq_(['/doesnt/exist'], retval.failed)
         assert not retval.succeeded
 
-
     @server()
     def test_get_should_not_use_windows_slashes_in_remote_paths(self):
         """
@@ -533,28 +571,22 @@ class TestFileTransfers(FabricTest):
             sftp = SFTP(env.host_string)
             eq_(sftp.glob(path), [path])
 
-
-
     #
     # put()
     #
-
     @server()
     def test_put_file_to_existing_directory(self):
         """
         put() a single file into an existing remote directory
         """
         text = "foo!"
-        local = self.path('foo.txt')
+        local = self.mkfile('foo.txt', text)
         local2 = self.path('foo2.txt')
-        with open(local, 'w') as fd:
-            fd.write(text)
         with hide('everything'):
             put(local, '/')
             get('/foo.txt', local2)
         eq_contents(local2, text)
 
-
     @server()
     def test_put_to_empty_directory_uses_cwd(self):
         """
@@ -573,7 +605,6 @@ class TestFileTransfers(FabricTest):
             get('foo.txt', local2)
         eq_contents(local2, text)
 
-
     @server()
     def test_put_from_empty_directory_uses_cwd(self):
         """
@@ -599,7 +630,6 @@ class TestFileTransfers(FabricTest):
         # Restore cwd
         os.chdir(old_cwd)
 
-
     @server()
     def test_put_should_accept_file_like_objects(self):
         """
@@ -617,7 +647,6 @@ class TestFileTransfers(FabricTest):
         # Sanity test of file pointer
         eq_(pointer, fake_file.tell())
 
-
     @server()
     @raises(ValueError)
     def test_put_should_raise_exception_for_nonexistent_local_path(self):
@@ -626,7 +655,6 @@ class TestFileTransfers(FabricTest):
         """
         put('thisfiledoesnotexist', '/tmp')
 
-
     @server()
     def test_put_returns_list_of_remote_paths(self):
         """
@@ -640,7 +668,6 @@ class TestFileTransfers(FabricTest):
             retval = put(f, p)
         eq_(retval, [p])
 
-
     @server()
     def test_put_returns_list_of_remote_paths_with_stringio(self):
         """
@@ -650,7 +677,6 @@ class TestFileTransfers(FabricTest):
         with hide('everything'):
             eq_(put(StringIO('contents'), f), [f])
 
-
     @server()
     def test_put_return_value_failed_attribute(self):
         """
@@ -662,12 +688,9 @@ class TestFileTransfers(FabricTest):
         eq_(["<StringIO>"], retval.failed)
         assert not retval.succeeded
 
-
-
     #
     # Interactions with cd()
     #
-
     @server()
     def test_cd_should_apply_to_put(self):
         """
@@ -682,7 +705,6 @@ class TestFileTransfers(FabricTest):
             put(local, f)
         assert self.exists_remotely('%s/%s' % (d, f))
 
-
     @server(files={'/tmp/test.txt': 'test'})
     def test_cd_should_apply_to_get(self):
         """
@@ -693,7 +715,6 @@ class TestFileTransfers(FabricTest):
             get('test.txt', local)
         assert os.path.exists(local)
 
-
     @server()
     def test_cd_should_not_apply_to_absolute_put(self):
         """
@@ -707,7 +728,6 @@ class TestFileTransfers(FabricTest):
         assert not self.exists_remotely('/tmp/test.txt')
         assert self.exists_remotely('/test.txt')
 
-
     @server(files={'/test.txt': 'test'})
     def test_cd_should_not_apply_to_absolute_get(self):
         """
@@ -718,7 +738,6 @@ class TestFileTransfers(FabricTest):
             get('/test.txt', local)
         assert os.path.exists(local)
 
-
     @server()
     def test_lcd_should_apply_to_put(self):
         """
@@ -734,7 +753,6 @@ class TestFileTransfers(FabricTest):
             put(f, '/')
         assert self.exists_remotely('/%s' % f)
 
-
     @server()
     def test_lcd_should_apply_to_get(self):
         """
